--[[ 

	Martin Karer / Sezz, 2014-2017
	InspectFrame Wealth Statistcs

--]]

-- Lua API
local pairs, gsub, strmatch, select, tinsert = pairs, string.gsub, string.match, select, table.insert;

-- World of Warcraft API/Constants
local _G, CreateFrame, GetComparisonStatistic, GetAchievementInfo, ClearAchievementComparisonUnit, IsAddOnLoaded, SetAchievementComparisonUnit = _G, CreateFrame, GetComparisonStatistic, GetAchievementInfo, ClearAchievementComparisonUnit, IsAddOnLoaded, SetAchievementComparisonUnit;
local BONUS_ROLL_REWARD_MONEY = BONUS_ROLL_REWARD_MONEY;

-- GLOBALS: GameTooltip, InspectFrame, InspectFrameTab1, InspectFrameInset, AchievementFrameComparison, ButtonFrameTemplate_HideButtonBar, PanelTemplates_SetNumTabs, INSPECTFRAME_SUBFRAMES, ElvUI

-----------------------------------------------------------------------------

-- Statistic IDs
local statistics = {
	328, -- Total gold acquired
	919, -- Gold earned from auctions
	334, -- Most gold ever owned
	329, -- Auctions posted
	332, -- Most expensive auction sold
	330, -- Auction purchases
	331, -- Most expensive bid on auction
};

-----------------------------------------------------------------------------

local FormatLargeNumber = function(amount)
	local formatted, k = amount;

	while true do  
		formatted, k = gsub(formatted, "^(-?%d+)(%d%d%d)", "%1.%2");
		if (k == 0) then
			break;
		end
	end

	return formatted;
end

local FormatLargeGoldAmount = function(amount)
	local gold, rest = strmatch(amount, "^(%d+)(.*)");

	if (gold and rest) then
		return (FormatLargeNumber(gold)..rest);
	else
		return amount;
	end
end

-----------------------------------------------------------------------------

local tabId = InspectFrame.numTabs + 1;

-- Create Tab Frame
local frame = CreateFrame("Frame", "InspectGoldFrame", InspectFrame);
frame:SetAllPoints(InspectFrame);
frame:EnableMouse(true);
frame:SetID(tabId);
frame:Hide();

frame:SetScript("OnShow", function(self)
	ButtonFrameTemplate_HideButtonBar(InspectFrame);
	SetAchievementComparisonUnit(InspectFrame.unit);

	if (IsAddOnLoaded("Blizzard_AchievementUI")) then
		AchievementFrameComparison:UnregisterEvent("INSPECT_ACHIEVEMENT_READY");
	end
end);

frame:SetScript("OnHide", function(self)
	ClearAchievementComparisonUnit();
	if (IsAddOnLoaded("Blizzard_AchievementUI")) then
		AchievementFrameComparison:RegisterEvent("INSPECT_ACHIEVEMENT_READY");
	end
	for i, button in pairs(self.buttons) do
		button.value:SetText("--");
	end
end);

frame:RegisterEvent("INSPECT_ACHIEVEMENT_READY");
frame:SetScript("OnEvent", function(self, event)
	for i, button in pairs(self.buttons) do
		button.value:SetText(FormatLargeGoldAmount(GetComparisonStatistic(button:GetID())));
	end
end);

tinsert(INSPECTFRAME_SUBFRAMES, frame:GetName());

-- Statistic Text
frame.buttons = {};

for i, id in pairs(statistics) do
	local button = CreateFrame("Button", "InspectGoldFrame"..(i + 1), frame, "SezzStatTemplate");
	button:SetID(id);
	button.text:SetText(select(2, GetAchievementInfo(id)));
	button.value:SetText("--");

	if (i > 1) then
		button:SetPoint("TOP", frame.buttons[i - 1], "BOTTOM", 0, 0);
	else
		button:SetPoint("TOPLEFT", InspectFrameInset, 4, -4);
	end

	frame.buttons[i] = button;
end

-- Create Tab Button
local button = CreateFrame("Button", "InspectFrameTab"..tabId, InspectFrame, "CharacterFrameTabButtonTemplate");
button:SetText(BONUS_ROLL_REWARD_MONEY);
button:SetID(tabId);
button:SetPoint("LEFT", _G["InspectFrameTab"..InspectFrame.numTabs], "RIGHT", -16, 0);
button:SetScript("OnEnter", function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(self:GetText(), 1, 1, 1);
end);
button:SetScript("OnLeave", InspectFrameTab1:GetScript("OnLeave"));
button:SetScript("OnClick", InspectFrameTab1:GetScript("OnClick"));

-- Add Tab
PanelTemplates_SetNumTabs(InspectFrame, tabId);

-- ElvUI Skinning
if (ElvUI) then
	local E = ElvUI[1];
	if (E.private.skins.blizzard.enable and E.private.skins.blizzard.character) then
		local S = E:GetModule("Skins");
		S:HandleTab(button);

		for _, button in pairs(frame.buttons) do
			button:StyleButton();
		end
	end
end
